/*
 * command.cc: Shell command class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.9
 * Last Change: April 05, 2010
 *
 *
 * Copyright (C) 2008, 2009, 2010
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "utils.h"
#include "command.h"

using std::vector;
using std::string;

namespace {
  // convert an integer to a phrase
  std::vector<std::string> init()
  {
    std::vector<std::string> ct(15);
    ct.at(0)  = "ONE";
    ct.at(1)  = "TWO";
    ct.at(2)  = "THREE";
    ct.at(3)  = "FOUR";
    ct.at(4)  = "FIVE";
    ct.at(5)  = "SIX";
    ct.at(6)  = "SEVEN";
    ct.at(7)  = "EIGHT";
    ct.at(8)  = "NINE";
    ct.at(9)  = "TEN";
    ct.at(10) = "ELEVEN";
    ct.at(11) = "TWELVE";
    ct.at(12) = "THIRTEEN";
    ct.at(13) = "FOURTEEN";
    ct.at(14) = "FIFTEEN";

    return ct;
  }

  const std::vector<std::string> English(init());

  // verbal(1, "time", true)  -> " ONE time"
  // verbal(4, "run", true)   -> " FOUR runs"
  // verbal(1, "time", false) -> ""
  // verbal(4, "run", false)  -> " ..... FOUR runs"
  string verbal(unsigned int ct, const string& msg, bool verbose)
  {
    if (ct == 0 || 15 < ct)
      {
	error("Command count out of range; please file a bug report");
	return ""; // never reach this, but silences compiler warning
      }

    else if (ct == 1)
      {
	if (!verbose)
	  return "";

	else // verbose
	  return  " " + English.at(ct-1) + " " + msg;
      }

    else
      return (verbose ? " " : " ..... ") + English.at(ct-1)
	+ " " + msg + "s"; // pluralize
  }

  // replace all occurrences of FILE in options with basename
  const string inflag("FILE");
  string insert_basename(string options, const string& basename)
  {
    string::size_type curr(options.find(inflag));
    while (curr != string::npos)
      {
	options.replace(curr, inflag.size(), basename);
	curr = options.find(inflag);
      }

    return options;
  }
} // end of namespace


command::~command() { }

// default implementation; latex, pdflatex override
std::string command::human_cmd(FILENAME fileroot) const
{
  return shell_cmd(fileroot);
}

// command base class
// Input: root name of a file
// Return value: none
// Action: run the command effected by shell_cmd() count() time(s)
int command::process(FILENAME fileroot) const
{
  const string cml(shell_cmd(fileroot));
  int status(0);
  unsigned int ct(0);

  while (status == 0 && ct++ < count())
    status = run_cmd(cml);

  if (status != 0)
    warning("Command \"" + cml + "\" failed after" + verbal(ct, "run", true));

  return status;
}

// return verbal human_cmd()
string command::print(FILENAME fileroot) const
{
  return human_cmd(fileroot) + verbal(count(), "time", false);
}


// latex (DISCARD options for security; mostly want to ignore \write 18)
latex::latex(const string& options, unsigned int count)
  : m_options("-no-shell-escape -interaction=batchmode "), m_count(count) { }

latex* latex::clone() const
{
  return new latex(*this);
}

// latex [options] fileroot.tex
string latex::shell_cmd(FILENAME fileroot) const
{
  // Do not allow the user to pass any options
  return "latex " + m_options + fileroot + ".tex";
}

string latex::human_cmd(FILENAME fileroot) const
{
  return "latex " + fileroot + ".tex";
}

const unsigned int latex::count() const
{
  return m_count;
}

// tex-to-dvi (0th entry) only
vector<bool> latex::validate(vector<bool> checklist) const
{
  checklist.at(0) = true;
  checklist.at(1) = checklist.at(2) = false;
  return checklist;
}


// pdflatex (DISCARD options for security)
pdflatex::pdflatex(const string& options, unsigned int count)
  : m_options("-no-shell-escape -interaction=batchmode "), m_count(count) { }

pdflatex* pdflatex::clone() const
{
  return new pdflatex(*this);
}

// pdflatex [options] fileroot.tex
string pdflatex::shell_cmd(FILENAME fileroot) const
{
  // Do not allow the user to pass any options
  return "pdflatex " + m_options + fileroot + ".tex";
}

string pdflatex::human_cmd(FILENAME fileroot) const
{
  return "pdflatex " + fileroot + ".tex";
}

const unsigned int pdflatex::count() const
{
  return m_count;
}

vector<bool> pdflatex::validate(vector<bool> checklist) const
{
  vector<bool> stages(3, true); // 1, 2, and 3
  return stages;
}


// dvips
dvips::dvips(const string& options, unsigned int count)
  : m_options(options), m_count(count)
{
  if (1 < count)
    warning("Multiple count requested for dvips");
}

dvips* dvips::clone() const
{
  return new dvips(*this);
}

// dvips -f [options] -o fr.ps fr.dvi
string dvips::shell_cmd(FILENAME fr) const
{
  return "dvips -f " + insert_basename(m_options, fr)
    + "-o " + fr + ".ps " + fr + ".dvi";
}

const unsigned int dvips::count() const
{
  return m_count;
}

vector<bool> dvips::validate(vector<bool> checklist) const
{
  if (checklist.at(0)) // tex-to-dvi done
    {
      checklist.at(1) = true; // 2 only
      checklist.at(2) = false;
    }

  return checklist;
}


// dvipdfm
dvipdfm::dvipdfm(const string& options, unsigned int count)
  : m_options(options), m_count(count)
{
  if (1 < count)
    warning("Multiple count requested for dvipdfm");
}

dvipdfm* dvipdfm::clone() const
{
  return new dvipdfm(*this);
}

// dvipdfm [options] fileroot.dvi
string dvipdfm::shell_cmd(FILENAME fileroot) const
{
  // User may pass options, but not the basename of the input file
  return "dvipdfm " + m_options + fileroot + ".dvi";
}

const unsigned int dvipdfm::count() const
{
  return m_count;
}

vector<bool> dvipdfm::validate(vector<bool> checklist) const
{
  if (checklist.at(0)) // tex-to-dvi done
    {
      checklist.at(1) = true; // 2 and 3
      checklist.at(2) = true;
    }

  return checklist;
}


// ps2pdf
ps2pdf::ps2pdf(const string& options, unsigned int count)
  : m_options(options), m_count(count)
{
  if (1 < count)
    warning("Multiple count requested for ps2pdf");
}

ps2pdf* ps2pdf::clone() const
{
  return new ps2pdf(*this);
}

// ps2pdf [options] fileroot.ps
string ps2pdf::shell_cmd(FILENAME fileroot) const
{
  // User may pass options, but not the basename of the input file
  return "ps2pdf " + m_options + fileroot + ".ps";
}

const unsigned int ps2pdf::count() const
{
  return m_count;
}

vector<bool> ps2pdf::validate(vector<bool> checklist) const
{
  if (checklist.at(1)) // dvi-to-ps done
    checklist.at(2) = true;

  return checklist;
}


// makeindex
makeindex::makeindex(const string& options, unsigned int count)
  : m_options(options), m_count(count)
{
  if (1 < count)
    warning("Multiple count requested for makeindex");
}

makeindex* makeindex::clone() const
{
  return new makeindex(*this);
}

// makeindex [options] fileroot.idx
string makeindex::shell_cmd(FILENAME fileroot) const
{
  // User specified no basename hook; operate on .idx file
  if (m_options.find("FILE") == std::string::npos)
    return "makeindex " + m_options + fileroot + ".idx";

  else // User must specify file to be processed
    return "makeindex " + insert_basename(m_options,fileroot);
}

const unsigned int makeindex::count() const
{
  return m_count;
}

vector<bool> makeindex::validate(vector<bool> checklist) const
{
  vector<bool> stages(3, false);
  return stages; // unset all flags
}
