/*
 * command.h: Shell command class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.0
 * Last Change: 08 June, 2008
 *
 *
 * Copyright (C) 2008
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <vector>
#include <string>

#include "typedefs.h"

// Abstraction of a compile command, e.g., latex, dvips, makeindex.
// Child classes must provide a constructor accepting an option string.
// The command_list class handles overall logic and string parsing, so
// command classes can be fairly naive.
class command {
public:
  virtual ~command();
  virtual command* clone() const = 0;

  // Return command string, e.g. "latex fileroot.tex", "makeindex fileroot.idx"
  virtual std::string shell_cmd(FILENAME fileroot) const = 0;
  virtual std::string human_cmd(FILENAME fileroot) const; // defaults to above

  // Number of times command is to be run consecutively
  virtual const unsigned int count() const = 0;

  // Input: root name of a file
  // Action: run the command effected by shell_cmd(), return exit status
  int process(FILENAME fileroot) const;

  // Compilation conceptually performs tex -(1)-> dvi -(2)-> ps -(3)-> pdf.
  // A "valid" sequence of commands performs these conversions in order.
  //
  // Validation status is maintained as an ordered triple of bools, assumed
  // to be one of FFF, TFF, TTF, or TTT, with T representing a completed
  // step and F representing an uncompleted step.
  //
  // Input: Validation status before we're applied
  // Return value: Same, after we're applied
  // Action: Mark or unmark one or more steps of the tex -> pdf compile
  //   process as "completed".
  // Examples: (i) If we represent "dvipdfm", our output is either FFF (if
  //   the tex -> dvi step is uncompleted in our input) or TTT (otherwise).
  //   (ii) If we represent "latex", our output is always TFF; any
  //   previously-completed dvi -> ps or ps -> pdf conversion is presumed
  //   stale after we've run.
  virtual std::vector<bool> validate(std::vector<bool>) const = 0;

  // return human_cmd() with repetition count
  std::string print(FILENAME fileroot) const;
}; // end of command class


//%%%%%%%%%%%%%%%%%%%%% Command Implementations %%%%%%%%%%%%%%%%%%%%%%

// command implementation declarations:
// latex, pdflatex, dvips, dvipdfm, ps2pdf, makeindex
class latex : public command {
public:
  latex(const std::string& options, unsigned int ct);
  latex* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  std::string human_cmd(FILENAME fileroot) const; // override
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};

class pdflatex : public command {
public:
  pdflatex(const std::string& options, unsigned int ct);
  pdflatex* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  std::string human_cmd(FILENAME fileroot) const; // override
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};

class dvips : public command {
public:
  dvips(const std::string& options, unsigned int ct);
  dvips* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};

class dvipdfm : public command {
public:
  dvipdfm(const std::string& options, unsigned int ct);
  dvipdfm* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};

class ps2pdf : public command {
public:
  ps2pdf(const std::string& options, unsigned int ct);
  ps2pdf* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};

class makeindex : public command {
public:
  makeindex(const std::string& options, unsigned int ct);
  makeindex* clone() const;

  std::string shell_cmd(FILENAME fileroot) const;
  const unsigned int count() const;

  std::vector<bool> validate(std::vector<bool>) const;

private:
  std::string m_options;
  unsigned int m_count;
};
