/*
 * command_list.cc: Command block class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.9
 * Last Change: April 05, 2010
 *
 *
 * Copyright (C) 2008, 2009, 2010
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <cstdlib>
#include <vector>
#include <iostream>

#include "line.h"
#include "utils.h"

#include "command.h"
#include "command_list.h"

using std::list;
using std::vector;
using std::string;

using std::cerr;
using std::endl;

// Utility functions used only in this file
namespace {
  // Action: split into strings of non-spaces separated by SPACE or TAB
  std::list<std::string> tokenize(string arg);

  // Input: Presumed command line
  // Return value: First space-delimited token
  string get_first_token(string);


  // Input: List of tokens from a presumed command line, and buckets to store
  //   the option string and number of times to repeat a command
  // Return value: none
  // Action: Starting with second token, collect successive tokens starting
  //   with "-" in "options", then examine next token to see if it has the
  //   form x<num> and (if so) store <num> in "count".
  // Remarks: This function potentially modifies its reference arguments.
  //   An option taking an argument is specified with --option:=value, as in
  //   "dvips -t:=8.5in,11in", which is interpreted as "dvips -t 8.5in,11in".
  void get_options(list<string> words, string& options, int& count);

  // Input: tail of a command line found in the LaTeX preamble
  // Return value: an int representing the number of times to run a command
  // Action: if arg = [space]x<int>[junk], return <int>; otherwise return 1.
  // Examples: "x4" -> 4, "x 2 %% run twice" -> 2
  // Purpose: Allows the compile chain to be abbreviated, with (e.g.)
  //   "latex [options] x3" signifying the command "latex [options]" is to
  //   be run three times.
  int get_cmd_count(string);

  // Input: Command line
  // Return value: Same, with anything following "#" removed
  string strip_shell_comment(const string&);

  // Allowed characters in user-suppied command lines:
  // Is character alphanumeric, period, comma, hyphen, or underscore?
  bool is_allowed(int);

  // Input: User-supplied de-commented command line
  // Return value: Same, with all punctuation except ".", "-", and "_" stripped
  string sanitize(string);
} // end of namespace


// For each line in cml, remove comments, check for recognized value
// and optional count. If found, push command(s) onto m_commands.
// If line not recognized and m_commands is non-empty, quit.
command_list::command_list(const list<line>& cml)
{
  CONST_LINE ln(cml.begin());

  while (ln != cml.end())
    {
      string first_token(get_first_token(de_comment((*ln).data())));

      // UPDATE logic in case of new commands.
      // Current organization favors readability at an infinitesimal
      // cost in efficiency
      if (first_token == "latex" ||
	  first_token == "pdflatex" ||
	  first_token == "dvips" ||
	  first_token == "dvipdfm" ||
	  first_token == "ps2pdf" ||
	  first_token == "makeindex")
	{
	  // parse command line
	  string cmd(sanitize(strip_shell_comment(de_comment((*ln).data()))));

	  list<string> words(tokenize(cmd));
	  list<string>::const_iterator curr(words.begin());

	  // default option string ("") and number of command repeatitions
	  string options;
	  int count(1);

	  // inspect remainder of line to update options, count
	  get_options(words, options, count);

	  if (first_token == "latex")
	    m_commands.push_back(new latex(options, count));

	  else if (first_token == "pdflatex")
	    m_commands.push_back(new pdflatex(options, count));

	  else if (first_token == "dvips")
	    m_commands.push_back(new dvips(options, count));

	  else if (first_token == "dvipdfm")
	    m_commands.push_back(new dvipdfm(options, count));

	  else if (first_token == "ps2pdf")
	    m_commands.push_back(new ps2pdf(options, count));

	  else if (first_token == "makeindex")
	    m_commands.push_back(new makeindex(options, count));
	}

      // already found compile history, and this line isn't a command
      else if (m_commands.size() != 0)
	break;

      ++ln; // increment "loop index"
    } // end of while loop
} // end of command_list::ctor


// N.B. copy ctor/assignment are private, deliberately not implemented
command_list::~command_list()
{
  for (list<command*>::const_iterator p = m_commands.begin();
       p != m_commands.end(); ++p)
    delete *p;
}


// Do we perform steps 1, 2, 3 in order? (tex -> dvi -> ps -> pdf)
bool command_list::validate(FILENAME fileroot) const
{
  vector<bool> checklist(3, false); // FFF

  // inspect commands serially, ask each to validate (update) checklist
  for (list<command*>::const_iterator p = m_commands.begin();
       p != m_commands.end(); ++p)
    checklist = (*p) -> validate(checklist);

  if (checklist.at(0) && checklist.at(1) && checklist.at(2))
    return true;

  // else print diagnostic and exit
  cerr << "The document asks to be compiled with the commands:" << endl;

  for (list<command*>::const_iterator p = m_commands.begin();
       p != m_commands.end(); ++p)
    cerr << ((*p) -> print(fileroot)) << endl;

  cerr << endl
       << "The following conversions are unperformed or stale:" << endl;

  if (!checklist.at(0))
    cerr << "  tex-to-dvi" << endl;

  if (!checklist.at(1))
    cerr << "  dvi-to-ps" << endl;

  if (!checklist.at(2))
    cerr << "  ps-to-pdf" << endl;

  error("Incomplete compile history");
  return false; // never reached, but disables compiler warning
}

int command_list::process(FILENAME fileroot) const
{
  int status(0);

  if (validate(fileroot)) // print diagnostic error message and exit on failure
    {
      list<command*>::const_iterator p(m_commands.begin());
      while (p != m_commands.end() && status == 0)
	status = (*p++) -> process(fileroot); // runs command repeatedly
    }

  return status;
}

// add each command count() times to list
list<string> command_list::print(FILENAME fileroot, bool verbal) const
{
  list<string> cmds;
  if (verbal)
    for (list<command*>::const_iterator p = m_commands.begin();
	 p != m_commands.end(); ++p)
      cmds.push_back((*p) -> print(fileroot)); // human-friendly description

  else
    for (list<command*>::const_iterator p = m_commands.begin();
	 p != m_commands.end(); ++p)
      for (unsigned int ct=0; ct < (*p) -> count(); ++ct)
	cmds.push_back((*p) -> shell_cmd(fileroot)); // shell command

  return cmds;
}


//////////////////// private utility functions ////////////////////

namespace {
  // Action: split into SPACE-delimited strings
  list<string> tokenize(string arg)
  {
    list<string> words;

    if (arg.size() == 0)
      words.push_back("");

    else
      {
	string::size_type curr(arg.find_first_not_of(" "));
	arg.erase(0, curr);

	while (arg.size() != 0)
	  {
	    curr = arg.find_first_of(" ");
	    words.push_back(arg.substr(0, curr));

	    arg.erase(0, curr);
	    arg.erase(0, arg.find_first_not_of(" "));
	  }
      }

    return words;
  }

  // Input: List of tokens from a presumed command line, and buckets to store
  //   the option string and number of times to repeat a command
  // Return value: none
  // Action: Discard first token (command name); inspect final or final two
  //   tokens for x<num> directive and (if found) store <num> in "count".
  //   Assign remaining tokens to "options".
  // Remarks: This function potentially modifies its reference arguments.
  void get_options(list<string> words, string& options, int& count)
  {
    // Need to look for options and/or "x<num>" directive?
    if (1 < words.size())
      {
	list<string>::iterator wp(words.begin());
	words.erase(wp); // remove first token

	wp = words.end();
	--wp; // final token

	string last(*wp);

	int tmp_ct(get_cmd_count(last));
	if (tmp_ct > 1) // last token is "x<num>"
	  {
	    count = tmp_ct;
	    words.erase(wp);
	  }

	else // inspect previous token
	  {
	    --wp;
	    last = (*wp) + last;
	    tmp_ct = get_cmd_count(last);
	    if (tmp_ct > 1) // last two tokens are "x <num>"
	      {
		count = tmp_ct;
		++wp; // final token
		wp = words.erase(wp);
		--wp;
		words.erase(wp);
	      }
	  }

	wp = words.begin();
	while (wp != words.end())
	  {
	    options = options + (*wp) + " ";
	    ++wp;
	  }

      } // finished collecting options and repetition directive
  }


  // Input: tail of a command line found in the LaTeX preamble
  // Return value: an int representing the number of times to run a command
  // Action: if arg = [space]x<int>[junk], return <int>; otherwise return 1
  int get_cmd_count(string arg)
  {
    arg.erase(0, arg.find_first_not_of(" ")); // remove initial white space
    if (arg.substr(0, 1) != "x" && arg.substr(0, 1) != "X")
      return 1;

    // else
    arg.erase(0, 1); // remove x, look for string of digits
    string::size_type begin(arg.find_first_of("0123456789"));
    string::size_type end(arg.find_first_not_of("0123456789"));

    if (begin == string::npos)
      return 1;

    if (end == string::npos)
      {
	arg = arg.substr(begin, arg.size() - begin + 1);
	return std::min(MAX_COUNT, atoi(arg.c_str()));
      }
    
    else if (begin < end)
      {
	arg = arg.substr(begin, end - begin + 1);
	return std::min(MAX_COUNT, atoi(arg.c_str()));
      }

    else
      return 1;
  }

  // Input: Presumed command line
  // Return value: First space-delimited token
  string get_first_token(string arg)
  {
    arg.erase(0, arg.find_first_not_of(" \t")); // remove initial white space
    return arg.substr(0, arg.find_first_of(" \t"));
  }

  // Input: Command line
  // Return value: Same, with anything following "#" removed
  string strip_shell_comment(const string& arg)
  {
    return arg.substr(0, arg.find_first_of("#"));
  }

  // Allowed characters in user-suppied command lines:
  // Is character alphanumeric, period, comma, hyphen, or underscore?
  bool is_allowed(int c)
  {
    return isalnum(c) || (c == '.') || (c == '-') || (c == '_')|| (c == ',');
  }

  // Input: User-supplied de-commented command line
  // Return value: Same, with ":=" -> " " and "unallowed" punctuation stripped
  string sanitize(string arg)
  {
    // Support legacy syntax; ":=" in a command line formerly signified a space
    arg = replace_all(arg, ":=", " ");

    // remove forbidden characters
    unsigned int curr(0);
    while (curr < arg.size())
      {
	if (!is_allowed(arg.at(curr)))
	  arg.at(curr) = ' ';
	++curr;
      }

    // For safety, forbid tokens from starting with "."
    arg = replace_all(arg, " .", " ");

    return arg;
  }
} // end of unnamed namespace
