/*
 * file.cc: Disk file class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.0
 * Last Change: 09 June, 2008
 *
 *
 * Copyright (C) 2008
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <fstream>

#include "utils.h"
#include "command_list.h"

#include "file.h"

using std::list;
using std::string;

namespace {
  std::string comment_block(string arg, unsigned int width)
  {
    string value("%% " + arg);
    if (value.size()+2 < width)
      {
	string end_pad(width - value.size() - 2, ' ');
	value += end_pad + "%%";
      }

    return value;
  }
} // end of namespace

file::file() { }

file::file(const line& L)
{
  m_lines.push_back(L);
}

file::file(const list<line>& lines)
  : m_lines(lines) { }

file::file(FILENAME filename)
{
  std::ifstream input(filename.c_str());
  if (!input)
    error("Can't open file \"" + filename + "\"");

  // else
  string linebuf;

  while (getline(input, linebuf))
    m_lines.push_back(line(linebuf));

  input.close();
}

// append line(s)
file& file::append(const line& L)
{
  m_lines.push_back(L);
  return *this;
}

file& file::append(list<string> strs)
{
  list<line> lines;
  for (list<string>::const_iterator p = strs.begin(); p != strs.end(); ++p)
    lines.push_back(line(*p));

  m_lines.splice(m_lines.end(), lines);
  return *this;
}

file& file::append(list<line> lines)
{
  m_lines.splice(m_lines.end(), lines);
  return *this;
}

file& file::prepend(list<line> lines)
{
  m_lines.splice(m_lines.begin(), lines);
  return *this;
}

// insert data before first match of f
file& file::prepend(const file& F, bool f(const string&))
{
  list<line>::iterator lp(m_lines.begin());

  while (lp != m_lines.end() && !f((*lp).data())) // skip non-matching lines
    ++lp;

  if (lp != m_lines.end()) // found a match
    {
      list<line> add(F.lines()); // copy data
      m_lines.splice(lp, add);
    }

  return *this;
}

// insert data after first match of f
file& file::append(const file& F, bool f(const string&))
{
  list<line>::iterator lp(m_lines.begin());

  while (lp != m_lines.end() && !f((*lp).data())) // skip non-matching lines
    ++lp;

  if (lp != m_lines.end())
    {
      ++lp;
      list<line> add(F.lines());
      m_lines.splice(lp, add);
    }

  return *this;
}

// write to FILENAME, ensuring DOS newlines
void file::write(FILENAME filename, bool DOS) const
{
  std::ofstream output(filename.c_str());
  if (!output)
    error("Can't open file \"" + filename + "\"");

  for (CONST_LINE lp = m_lines.begin(); lp != m_lines.end(); ++lp)
    output << (*lp).print(DOS);

  output.close();
}

// format by padding front and back with "%", to specified total width
void file::format(unsigned int width)
{
  if (m_lines.size() == 0 || width < 4)
    return;

  // else
  string leader("% " + string(width-4, '%') + " %"); // % %%%...% %
  string sublead("%" + string(width-2, ' ') + "%");  // %   ...   %

  for (LINE lp = m_lines.begin(); lp != m_lines.end(); ++lp)
    (*lp) = (*lp).format(width);

  m_lines.push_front(sublead);
  m_lines.push_front(leader);

  m_lines.push_back(sublead);
  m_lines.push_back(leader);
  m_lines.push_back(string(""));
}

// remove initial and trailing blank line(s)
void file::de_pad()
{
  list<line>::iterator lp(m_lines.begin());

  while (lp != m_lines.end() && is_blank((*lp).data()))
    lp = m_lines.erase(lp); // implicit increment

  if (m_lines.size() != 0) // non-blank lines remain
    {
      lp = m_lines.end();
      --lp;
      while (is_blank((*lp).data())) // needn't check for out-of-range
	{
	  lp = m_lines.erase(lp);
	  --lp;
	}
    }
}


// private, assumes lp is in range
void file::replace_line(const file& data, list<line>::iterator lp)
{
  lp = m_lines.erase(lp); // delete line

  for (list<line>::const_iterator ln = data.m_lines.begin();
       ln != data.m_lines.end(); ++ln)
    {
      lp = m_lines.insert(lp, *ln);
      ++lp; // skip past line just added
    }
}


bool file::replace_line(const file& data, placeholder_type ph)
{
  list<line>::iterator lp(m_lines.begin());

  while (lp != m_lines.end() && (*lp).type() != ph) // skip non-matching lines
    ++lp;

  if (lp == m_lines.end()) // no match
    return false;

  // else
  replace_line(data, lp);
  return true;
}


// pass lines successively to f, return first != 0 value
int file::select(int f(const string&), bool forward) const
{
  CONST_LINE lp(forward ? m_lines.begin() : m_lines.end());
  int value(0);

  if (forward)
    while (lp != m_lines.end() && value == 0)
      {
	value = f((*lp).data());
	++lp;
      }

  else
    while (lp != m_lines.begin() && value == 0)
      {
	--lp;
	value = f((*lp).data());
      }

  return value;
}

string file::select(string f(const string&)) const
{
  string value;
  CONST_LINE lp(m_lines.begin());

  while(lp != m_lines.end() && value.size() == 0)
    {
      value = f((*lp).data());
      ++lp;
    }

  return value;
}

// extract from first line satisfying fm to next line satisfying to
list<line> file::extract(bool fm(const string&),
			 bool to(const string&)) const
{
  list<line> value;
  CONST_LINE lp(m_lines.begin());

  while(lp != m_lines.end() && !fm((*lp).data()))
    ++lp;

  while(lp != m_lines.end() && !to((*lp).data()))
    {
      value.push_back(*lp);
      ++lp;
    }

  return value;
}


// Use f to extract information from our lines
list<line> file::summarize(string f(const string&)) const
{
  list<line> value;
  for (CONST_LINE lp = m_lines.begin(); lp != m_lines.end(); ++lp)
    {
      string tmp(f((*lp).data()));

      if (tmp.size() != 0)
	value.push_back(line(tmp));
    }

  return value;
}


const list<line>& file::lines() const
{
  return m_lines;
}

// look for "PDF Pages...", substitute number of pages, preserving formatting
void file::update_pdf_pages(int n)
{
  LINE lp(m_lines.begin());

  while (lp != m_lines.end() && !pdf_pagecount((*lp).data()))
    ++lp;

  if (lp == m_lines.end()) // no match
    return;

  // else
  string new_nbr(rep(n));

  // line presumed to contain *one* contiguous string of digits
  string tmp((*lp).data());
  string::size_type from(tmp.find_first_of("0123456789"));

  if (from == string::npos) // found "PDF Pages..." but no number
    *lp = line("%% PDF Pages: " + new_nbr);

  else
    {
      string::size_type to(tmp.find_last_of("0123456789"));
      string old_nbr(tmp.substr(from, to - from + 1));

      // pad shorter string with one space; if counts mismatch by more
      // than one digit, prettily formatting this line isn't a concern
      if (old_nbr.size() < new_nbr.size())
	old_nbr  = old_nbr + " ";

      else if (new_nbr.size() < old_nbr.size())
	new_nbr  = new_nbr + " ";

      tmp.replace(from, new_nbr.size(), new_nbr);

      *lp = line(tmp);
    }
} // end of file::update_pdf_pages


// tag = "tex" or "pdf"; see update_pgnames()
void file::update_pdf_names(const string& tag)
{
  LINE lp(m_lines.begin());

  while (lp != m_lines.end() && !is_pgnames((*lp).data()))
    ++lp;

  if (lp == m_lines.end()) // no match
    return;

  // else *lp = 
  // "***** This file should be named [...].zip *****"
  *lp = line(update_pgnames((*lp).data(), tag));
}

void file::update_history(FILENAME fileroot, const string& distro,
			  const command_list& cml)
{
  LINE lp(m_lines.begin());

  // skip non-comments
  while (lp != m_lines.end() && !is_comment((*lp).data()))
    ++lp;

  // skip comments
  while (lp != m_lines.end() && is_comment((*lp).data()))
    ++lp;

  if (lp == m_lines.end()) // no match
    return;

  // else pretty-format the compile date and command block
  --lp; // examine last line of comment block
  unsigned int wd((*lp).data().size()); // width of comment block

  m_lines.insert(lp, line(comment_block("", wd)));
  m_lines.insert(lp, line(comment_block(month()+": pglatex.", wd)));
  m_lines.insert(lp, line(comment_block("  Compile this project with:", wd)));

  list<string> cmds(cml.print(fileroot, true)); // print verbally

  for (list<string>::const_iterator p=cmds.begin(); p!=cmds.end(); ++p)
    m_lines.insert(lp, line(comment_block("  " + *p, wd)));

  m_lines.insert(lp, line(comment_block("", wd)));
  m_lines.insert(lp, line(comment_block("  " + distro, wd)));
  m_lines.insert(lp, line(comment_block("", wd)));
}
