/*
 * file.h: Disk file class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.0
 * Last Change: 09 June, 2008
 *
 *
 * Copyright (C) 2008
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef PGL_FILE
#define PGL_FILE

#include <list>
#include <string>

#include "constants.h"
#include "typedefs.h"

#include "line.h"

class command_list;

// abstraction in memory of a general disk file
class file {
public:
  file();
  file(const line&);            // implicitly promote a line to a file
  file(const std::list<line>&); // build from list of lines
  file(FILENAME);               // read from disk

  // append/prepend line(s)
  file& append(const line&);
  file& append(std::list<std::string>);
  file& append(std::list<line>);
  file& prepend(std::list<line>);

  // insert file contents before/after first match of f
  file& prepend(const file&, bool f(const std::string&));
  file& append(const file&, bool f(const std::string&));

  // write to FILENAME, ensuring DOS newlines
  void write(FILENAME, bool DOS=true) const;

  // format by padding front and back with "%", to specified total width
  void format(unsigned int width=MAX_FILE_WIDTH);

  // remove initial and trailing blank line(s)
  void de_pad();

  // replace matching line with contents of file, return success status
  bool replace_line(const file&, placeholder_type);

  // return first non-zero value of f; forward=false searches backward
  int select(int f(const std::string&), bool forward=true) const;

  // return first non-empty value of f
  std::string select(std::string f(const std::string&)) const;

  // extract from first line satisfying fm to next line satisfying to
  std::list<line> extract(bool fm(const std::string&),
			  bool to(const std::string&)) const;

  // Use f to process information from our lines
  std::list<line> summarize(std::string f(const std::string&)) const;

  const std::list<line>& lines() const;

  // specialized functions not handled by general functions above
  void update_pdf_pages(int);

  // tag should be "tex" or "pdf", and is passed to update_pgnames()
  void update_pdf_names(const std::string& tag);

  void update_history(FILENAME fileroot, const std::string& distro,
		      const command_list&);

private:
  std::list<line> m_lines;

  // delete line and insert file contents
  void replace_line(const file&, std::list<line>::iterator);
}; // end of file class

#endif /* PGL_FILE */
