/*
 * input_file.cc: LaTeX source file class for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.5
 * Last Change: September 05, 2009
 *
 *
 * Copyright (C) 2008, 2009
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <cstdlib>
#include <iostream>
#include <fstream>

#include "utils.h"
#include "input_file.h"

using std::list;
using std::string;

// read-only represention of file created by PG posting program
// UPDATE constructor logic if stanza types change

// read named file into memory, then search for stanza separators:
// boilerplate, credits, PPer comments, latex source/log files, and license
input_file::input_file(FILENAME filename)
  : m_license(m_lines.end()), m_warnings()
{
  std::ifstream input(filename.c_str());
  if (!input)
    error("Can't open file \"" + filename + "\"");

  // else
  string linebuf;
  bool found_listfiles(false);

  // read named disk file, look for \listfiles line
  while (getline(input, linebuf))
    {
      m_lines.push_back(line(linebuf));
      found_listfiles = found_listfiles || is_listfiles(linebuf);
    }

  input.close();

  if (m_lines.size() == 0)
    error("\"" + filename + "\" contains no data!");

  // else
  if (!found_listfiles) // add \listfiles after \documentclass
    {
      m_warnings.push_back
	(line("Unable to compare the uploader's package versions with ours."));

      m_warnings.push_back
	(line("Please ask the uploader to use \"\\listfiles\" in the future."));

      LINE lp_tmp(m_lines.begin());
      seek_past(lp_tmp, documentclass, "No \\documentclass line found");
      m_lines.insert(lp_tmp, line("\\listfiles"));
    }

  // else
  // set separators; depends on overall expected structure of file
  LINE lp(m_lines.begin());

  seek_past(lp, end_boilerplate, "File ends after boilerplate");
  m_header = lp; // matching line is part of previous stanza

  seek_past(lp, start_latex, "File ends after credits");

  m_credits = --lp; // matching line not part of previous stanza

  seek_past(lp, end_document, "No \\end{document} found");
  // lp points after \end{document}; look for lprep config

  while (lp != m_lines.end() && is_blank((*lp).data())) // skip blank lines
    ++lp;

  // lp == first non-blank after \end{document}
  if (lp != m_lines.end() && is_lprep((*lp).data())) // found "###..."?
    {
      do {
	++lp;
      }
      while (lp != m_lines.end() && !is_lprep((*lp).data()));

      if (lp != m_lines.end())
	++lp; // successor of second "###..." line
    }

  // mark end of latex file; provisionally set end of log file
  m_logfile = m_latex = lp;

  if (lp == m_lines.end())
    {
      m_warnings.push_back(line("No license found"));
      print_warnings();
      exit(1);
    }

  // else input file might not contain a log file, so we'll search manually
  while (lp != m_lines.end() && !end_logfile((*lp).data()))
    ++lp;

  if (lp != m_lines.end()) // saw "Output written on..."
    {
      m_logfile = ++lp;

      if (lp != m_lines.end() && pdf_logfile((*lp).data()))
	{
	  // seek next blank line, presumably marking end of PDF log file
	  int pdf_log_lines(0);
	  while (lp != m_lines.end() && !is_blank((*lp).data()))
	    {
	      ++lp;
	      ++pdf_log_lines;
	    }

	  m_logfile = lp;
	}

      if (lp == m_lines.end())
	error("No license found");
    }

  else
    std::cerr << "Warning: No PPer log file found" << std::endl;

  // m_license already set to m_lines.end()


  // Get PPer's comment block; may assume file is well-formed at this stage
  lp = m_credits; // start of latex stanza
  while (lp != m_lines.end() && !is_comment((*lp).data())) // skip non-comments
    ++lp;

  if (lp == m_lines.end())
    error("No PPer comment block found");

  // else
  m_PP_begin = lp; // first "%%" line after start of latex file

  while (lp != m_lines.end() && is_comment((*lp).data())) // skip "%%" lines
    ++lp;

  m_PP_end = lp;
} // end of input_file constructor


// return stanzas
list<line> input_file::header()   const
{
  return extract(m_lines.begin(), m_header);
}

list<line> input_file::credits()  const
{
  return extract(m_header, m_credits);
}

list<line> input_file::comments() const
{
  return extract(m_PP_begin, m_PP_end);
}

list<line> input_file::latex()    const
{
  return extract(m_credits, m_latex);
}

list<line> input_file::logfile()  const
{
  return extract(m_latex, m_logfile);
}

list<line> input_file::license()  const
{
  return extract(m_logfile, m_license);
}

void input_file::print_warnings()  const
{
  if (m_warnings.size() != 0)
    {
      std::cerr << "@@@@@ Input file had the following problem(s): @@@@@\n";
      for (std::list<line>::const_iterator lp=m_warnings.begin();
	   lp != m_warnings.end(); ++lp)
	std::cerr << (*lp).data() << "\n";
    }
}


// private implementation functions
// find successor of first line satisfying f(*lp),
// exit if m_lines.end() reached
void input_file::seek_past(LINE& lp, bool f(const string&),
			   const std::string& err_msg) const
{
  // find first matching line (or EOF)
  while (lp != m_lines.end() && !f((*lp).data()))
    ++lp;

  // skip past (if not at end)
  if (lp != m_lines.end())
    ++lp;

  if (lp == m_lines.end()) // no match, or matching line ends file
    {
      m_warnings.push_back(line(err_msg));
      print_warnings(); // print accumulated warnings
      exit(1);    // and bomb out
    }
}

list<line> input_file::extract(CONST_LINE begin, CONST_LINE end) const
{
  list<line> stanza;

  for (CONST_LINE lp = begin; lp != end; ++lp)
    stanza.push_back(*lp);

  return stanza;
}
