/*
 * pglatex_post: A program to facilitate whitewashing LaTeX projects
 * at Project Gutenberg
 *
 * Version 1.1.0
 * Last Change: June 08, 2008
 *
 *
 * Copyright (C) 2008
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * Usage: pglatex_post <ebook>-WW.tex
 *
 * <ebook>-WW.tex is the LaTeX source file manually verified by the
 * WWer, and incorporating the PG boilerplate, credits, and license
 * inserted by the companion program pglatex_check.
 *
 * Action: The program updates the preamble's PDF page count, renames
 * files appropriately (see below), and zips them for posting.
 */

#include <iostream>
#include <string>

#include "command_list.h"
#include "utils.h"

#include "file.h"
#include "input_file.h"

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/*
 * (1) Read in latex file <ebook>-WW.tex, extract compile history.
 * (2) Compile <ebook>-WW.tex, update PDF page count.
 * (3) Rename source and PDF files to <ebook>-t.tex and <ebook>-pdf.pdf.
 *   Zip the LaTeX file and images to <ebook>-t.zip. Zip the PDF file to
 *   <ebook>-pdf.zip.
 * (4) Clean up, print summary message.
 */

int main(int argc, char* argv[])
{
  if (argc != 2)
    error("Usage: " + std::string(argv[0]) + " <ebook>-WW.tex");

  std::string filename(argv[1]); // presumably <ebook>-WW.tex

  check_extension(filename, "-WW.tex"); // bomb out on mismatch

  file infile(filename); // else load file

  command_list history(infile.lines()); // get command block

  // get file rootname
  const std::string ebook(filename.substr(0, filename.find_first_of("-")));

  const std::string WW_latex(ebook + "-t");
  const std::string WW_pdf  (ebook + "-pdf");

  const std::string WW_srcdir(ebook + "-t");
  const std::string WW_pkgdir(ebook);

  // compile
  infile.write(WW_latex + ".tex");
  if (history.process(WW_latex) != 0)
    error("The LaTeX compile failed; please consult the WWer manual");

  // read log file
  file WW_logfile(WW_latex + ".log");
  const int WW_pages(WW_logfile.select(page_count, false)); // search backward

  // update page count
  infile.update_pdf_pages(WW_pages);

  // append most recent log file
  infile.append(WW_logfile.lines());

  // write updated file back out
  infile.write(WW_latex + ".tex");

  // create <ebook> directory; populate with:
  // 1a. <ebook>-t directory, containing sources
  // 1b. <ebook>-t.zip file, zipped <ebook>-t
  // 2a. <ebook>-pdf.pdf file, compiled PDF
  // 2b. <ebook>-pdf.zip file, zipped PDF
  run_cmd("mkdir -p " + WW_pkgdir);
  run_cmd("mkdir -p " + WW_srcdir); // = WW_latex, but conceptually distinct

  // store images; run silently if no images
  run_cmd("cp -rp images " + WW_srcdir + " 2>/dev/null");
  run_cmd("cp -p "  + WW_latex  + ".tex " + WW_srcdir);

  run_cmd("zip -r " + WW_latex  + ".zip " + WW_srcdir); // zip sources;
  run_cmd("mv "  + WW_srcdir + " "     + WW_pkgdir); // store sources
  run_cmd("mv "  + WW_latex  + ".zip " + WW_pkgdir); // and zipped sources

  // rename PDF file <ebook>-t.pdf -> <ebook>-pdf.pdf
  run_cmd("mv -f " + WW_latex + ".pdf " + WW_pdf + ".pdf");

  // zip PDF file
  run_cmd("zip " + WW_pdf + ".zip " + WW_pdf + ".pdf");

  run_cmd("mv "  + WW_pdf + ".pdf " + WW_pkgdir); // store PDF
  run_cmd("mv "  + WW_pdf + ".zip " + WW_pkgdir); // and zipped PDF

  run_cmd("zip -r " + ebook + ".zip " + WW_pkgdir); // zip up package directory

  // remove LaTeX temporary files
  clean_tempfiles(WW_latex);
  //  run_cmd("rm -f " + WW_latex + ".tex");

  std::cout << std::endl
	    << "\tCongratulations: Everything seems to have worked!"
	    << std::endl
	    << "Please inspect "
	    << WW_pkgdir << "/" << WW_pdf
	    << ".pdf visually before uploading the zip files."
	    << std::endl;

  return(0);
} // end of main
