/*
 * utils.cc: Utility functions for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.9
 * Last Change: April 4, 2010
 *
 *
 * Copyright (C) 2008, 2009, 2010
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <cstdlib>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <locale>
#include <ctime>

#include "utils.h"

using std::string;

// functor for converting strings to lowercase
namespace {
  class ToLower
  {
  public:
    ToLower(std::locale const& loc)
      : m_loc(loc) { }

    char operator() (char c) const
    {
      return std::tolower(c, m_loc);
    }

  private:
    std::locale const& m_loc;
  };

  static const ToLower down(std::locale::classic());
} // end of namespace


void debug_msg(const string& msg)
{
  std::cerr << msg << std::endl;
}

// Action: Print warning string
void warning(const string& msg)
{
  std::cerr << "Warning: " << msg << std::endl;
}

// Action: Print error string and exit with error status
void error(const string& msg)
{
  std::cerr << "Error: " << msg << std::endl;
  exit(1);
}

// Action: execute system command; arg passed as string, not const char*
int run_cmd(const string& arg)
{
  return std::system(arg.c_str());
}

// remove file, return status
int rm_file(const string& filename)
{
  return run_cmd("rm -f " + filename);
}

// move file to file-1.bak (etc.), keeping existing numbered backups
int mv_safe(const string& filename, unsigned int version)
{
  std::string curr(version == 0 ? filename : ""); // caller-specified name

  if (version != 0) // we're in a recursive call; adjust current filename
    {
      std::stringstream buf;
      buf << filename << "-" << version << ".bak";
      curr = buf.str();
    }

  std::stringstream obuf;
  obuf << filename << "-" << version+1 << ".bak"; // new filename
  std::string next(obuf.str());

  if (run_cmd("test -f " + next) == 0) // file exists, recurse
    mv_safe(filename, version+1);

  return run_cmd("mv " + curr + " " + next);
}

// remove unneeded files
void clean_tempfiles(FILENAME fileroot)
{
  // clean up; runs silently even if files don't exist
  rm_file(fileroot + ".aux");
  rm_file(fileroot + ".dvi");
  rm_file(fileroot + ".idx");
  rm_file(fileroot + ".ilg");
  rm_file(fileroot + ".ind");
  //  rm_file(fileroot + ".log");
  rm_file(fileroot + ".out");
  rm_file(fileroot + ".ps");
  rm_file(fileroot + ".toc");
}


string month()
{
  const int max_len(15); // "September 2006"
  char buf[max_len];
  time_t t=time(0);

  strftime(buf, max_len, "%B %Y", gmtime(&t));
  return string(buf);
}


// Action: verify filename has specified extension, exit with error if not
void check_extension(const string& filename, const string& extn)
{
  unsigned int sz(extn.size());
  unsigned int fsz(filename.size());

  if (fsz <= sz || filename.substr(fsz - sz, sz) != extn)
    error("Input filename must have extension \"" + extn + "\"");
}

// Action: Convert int to string
string rep(int n)
{
  std::stringstream buf;
  buf << n;
  return buf.str();
}


//%%%%%%%% String manipulation %%%%%%%%

// Action: remove surrounding white space and (La)TeX comment chars
string de_comment(const string& arg)
{
  string::size_type begin(arg.find_first_not_of(" \t%"));
  string::size_type end(arg.find_last_not_of(" \t%"));

  return begin == end ? "" : arg.substr(begin, end - begin + 1);
}

// Action: remove surrounding white space and asterisks
string de_star(const string& arg)
{
  string::size_type begin(arg.find_first_not_of(" \t*"));
  string::size_type end(arg.find_last_not_of(" \t*"));

  return begin == end ? "" : arg.substr(begin, end - begin + 1);
}

// Action: Remove any trailing CR or LF (strip DOS, *nix, or MacOS 9 newline)
string de_newline(const string& arg)
{
  unsigned int sz(arg.size());
  if (sz != 0 && arg.substr(sz - 1, 1) == "\n")
    --sz;

  if (sz != 0 && arg.substr(sz - 1, 1) == "\r")
    --sz;

  return arg.substr(0, sz);
}

// Action: Replace all occurrences in arg of "from" with "to", return result
string replace_all(string arg, const string& from, const string& to)
{
  std::string::size_type pos(arg.find(from));
  while (pos != std::string::npos)
    {
      arg.replace(pos, from.size(), to);
      pos = arg.find(from);
    }
  return arg;
}

string lowercase(string s)
{
  std::transform(s.begin(), s.end(), s.begin(), down);
  return s;
}

string lowercase(const char* arg)
{
  return lowercase(string(arg));
}

// Action: Replace [-8].txt and [-8].zip in the license stanza
// by -t.tex and -t.zip (tag == "tex") or
// by -pdf.pdf and -pdf.zip (tag ="pdf")
std::string update_pgnames(std::string arg, const std::string& tag)
{
  if (is_pgnames(arg))
    {
      if (arg.find("-8.txt") != std::string::npos &&
	  arg.find("-8.zip") != std::string::npos)
	{
	  if (tag == "pdf")
	    {
	      arg.replace(arg.find("-8.txt"), 6, "-pdf.pdf");
	      arg.replace(arg.find("-8.zip"), 6, "-pdf.zip");
	    }

	  else if (tag == "tex")
	    {
	      arg.replace(arg.find("-8.txt"), 6, "-t.tex");
	      arg.replace(arg.find("-8.zip"), 6, "-t.zip");
	    }
	}

      else if (arg.find(".txt") != std::string::npos &&
	  arg.find(".zip") != std::string::npos)
	{
	  if (tag == "pdf")
	    {
	      arg.replace(arg.find(".txt"), 4, "-pdf.pdf");
	      arg.replace(arg.find(".zip"), 4, "-pdf.zip");
	    }

	  else if (tag == "tex")
	    {
	      arg.replace(arg.find(".txt"), 4, "-t.tex");
	      arg.replace(arg.find(".zip"), 4, "-t.zip");
	    }
	}
    }

  // no change on false positive match
  return arg;
}



//////////////////// Matching criteria ////////////////////
// Input: text string
// Output: match? (T/F)

// starts with "%%"
bool is_comment(const string& arg)
{
  string::size_type start(arg.find_first_not_of(" \t"));
  return (start != string::npos && arg.substr(start, 2) == "%%");
}

bool not_comment(const string& arg)
{
  return !is_comment(arg);
}

// entirely white space
bool is_blank(const string& arg)
{
  return (arg.find_first_not_of(" \t") == string::npos);
}

// \listfiles, possibly padded with white space/followed by a comment
bool is_listfiles(const string& arg)
{
  // first non-whitespace character
  string::size_type begin(arg.find_first_not_of(" \t"));

  if (begin == string::npos ||   // "empty" line?
      arg.size() - begin < 10 || // too short?
      arg.substr(begin, 10) != "\\listfiles")
    return false;

  // first comment character or npos
  string::size_type end(arg.find_first_of("%\r\n"));

  if (end == string::npos)
    end = arg.size();

  // end points to non-% character; get first non-space to just before %
  string tmp(arg.substr(begin, end - begin));

  // effectively:
  // end = tmp.find_last_not_of(" \t");
  // return (tmp.substr(0, end + 1) == "\\listfiles");
  return (tmp.substr(0, tmp.find_last_not_of(" \t") + 1) == "\\listfiles");
}

// "***** This file should be named [...].zip *****"
bool is_pgnames(const std::string& arg)
{
  return
    (arg.substr(0, 32) == "***** This file should be named ") &&
    (arg.substr(arg.size()-10, 10) == ".zip *****");
}

// "PDF Pages: ..."
bool pdf_pagecount(const string& arg)
{
  return (lowercase(de_comment(arg)).substr(0, 9) == "pdf pages");
}

// beginning of LaTeX code: first char is "%" or "\"
bool start_latex(const string& arg)
{
  return (arg.substr(0,1) == "%" || arg.substr(0,1) == "\\");
}

bool documentclass(const string& arg)
{
  return (arg.substr(0, 14) == "\\documentclass");
}

// end of boilerplate; arg has been de_commented, de_starred
bool end_boilerplate(const string& arg)
{
  string tmp(de_star(arg));
  return (tmp.substr(0, 30) == "START OF THE PROJECT GUTENBERG" ||
	  tmp.substr(0, 31) == "START OF THIS PROJECT GUTENBERG");
}

// LaTeX begin/end of document; initial white space OK
bool begin_document(const string& arg)
{
  if (is_blank(arg))
    return false;
  else
    return arg.substr(arg.find_first_not_of(" \t"), 16) == "\\begin{document}";
}

bool end_document(const string& arg)
{
  if (is_blank(arg))
    return false;

  else
    return (arg.substr(arg.find_first_not_of(" \t"), 14) == "\\end{document}");
}

// lprep configuration delimiter
bool is_lprep(const string& arg)
{
  return (arg.substr(0,3) == "###");
}

// end of log file
bool end_logfile(const string& arg)
{
  return (arg.substr(0, 18) == "Output written on ");
}

// additional logfile lines written by pdflatex on some platforms
bool pdf_logfile(const string& arg)
{
  return (arg.substr(0, 15) == "PDF statistics:");
}

// CAUTION: Dependent upon LaTeX's output strings
bool file_list(const std::string& arg)
{
  return arg.find("*File List*") != string::npos;
}

bool end_file_list(const std::string& arg)
{
  return arg.find("***********") != string::npos;
}


// Select and process log file lines for WWer's log report;
// UPDATE to log lines satisfying additional criteria
string underfull(const string& arg)
{
  return (arg.find("Underfull") != string::npos) ? arg : "";
}

string overfull(const string& arg)
{
  return (arg.find("Overfull") != string::npos) ? arg : "";
}

string latex_err(const string& arg)
{
  return (arg.find("LaTeX Error") != string::npos) ? arg : "";
}

string missing_pkg(const string& arg)
{
  return (arg.find("LaTeX Error: File") != string::npos) ? arg : "";
}

// Action: look for "Output written on ... (p pages, B bytes).", return p
int page_count(const string& arg)
{
  if (arg.substr(0, 18) == "Output written on ") // appropriately cautious?
    {
      string::size_type begin(arg.find_last_of("("));
      string::size_type end(arg.find_first_not_of("0123456789", ++begin));

      return atoi(arg.substr(begin, end - begin).c_str());
    }
  else
    return 0;
}
