/*
 * utils.h: Utility functions for pglatex_check and
 * pglatex_post, programs to compile LaTeX documents at
 * Project Gutenberg
 *
 * Version 1.1.9
 * Last Change: April 04, 2010
 *
 *
 * Copyright (C) 2008, 2009, 2010
 *
 * Andrew D. Hwang (DP: adhere)
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * pglatex is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pglatex is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pglatex; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <string>

#include "typedefs.h"

//%%%%%%%%%%%%%%%%%%% Global Function Declarations %%%%%%%%%%%%%%%%%%%

//%%%%%%%% Miscellaneous utilities %%%%%%%%

void debug_msg(const std::string& msg);

// Action: Print error string
void warning(const std::string& msg);

// Action: Print error string and exit with error status
void error(const std::string& msg);

// Input: command string
// Return value: exit status of command
// Action: execute system command (N.B. arg passed as string, not const char*)
int run_cmd(const std::string&);

// remove file, return status
int rm_file(const std::string&);

// move file to file-1.bak (etc.), keeping existing numbered backups
int mv_safe(const std::string&, unsigned int version=0);

// Action: remove aux, dvi, idx, ilg, ind, log, out, ps, toc files
void clean_tempfiles(FILENAME fileroot);

// Action: return month and year (e.g. "September 2006")
std::string month();

// Action: Verify filename has specified extension, exit with error if not
void check_extension(const std::string& filename, const std::string& extn);

// Action: Convert int to string
std::string rep(int);


//%%%%%%%% String manipulation %%%%%%%%

// Action: remove surrounding white space and (La)TeX comment chars (%)
std::string de_comment(const std::string& arg);

// Action: remove surrounding white space and asterisks (*)
std::string de_star(const std::string& arg);

// Action: Remove any trailing CR or LF (strip DOS, *nix, or MacOS 9 newline)
std::string de_newline(const std::string& arg);

// Action: Replace all occurrences in arg of "from" with "to", return result
std::string replace_all(std::string arg,
			const std::string& from, const std::string& to);

// Action: Convert input to lowercase in std::locale::classic
std::string lowercase(std::string);
std::string lowercase(const char*);

// Action: Replace -8.txt and -8.zip by -t.tex and -t.zip (tag = "tex")
// or by -pdf.pdf and -pdf.zip (tag = "pdf")
std::string update_pgnames(std::string, const std::string& tag);

//////////////////// String classification ////////////////////

//%%%%%%%% Matching criteria %%%%%%%%
// Input: text string
// Output: match? (T/F)

// starts with "%%" or not; for PPer's comment block
bool is_comment(const std::string& arg);
bool not_comment(const std::string& arg);

// entirely white space
bool is_blank(const std::string& arg);

// \listfiles
bool is_listfiles(const std::string& arg);

// Depends on format of PG license text:
// "***** This file should be named [...].zip *****"
bool is_pgnames(const std::string& arg);

// "PDF Pages..."
bool pdf_pagecount(const std::string& arg);

// beginning of LaTeX code
bool start_latex(const std::string& arg);

// documentclass line
bool documentclass(const std::string& arg);

// end of boilerplate
bool end_boilerplate(const std::string& arg);

// LaTeX begin/end of document
bool begin_document(const std::string& arg);
bool   end_document(const std::string& arg);

// lprep configuration delimiter
bool is_lprep(const std::string& arg);

// end of log file
bool end_logfile(const std::string& arg);

// additional logfile lines written by pdflatex on some platforms
bool pdf_logfile(const std::string& arg);

// begin/end of file list in log file
bool file_list(const std::string& arg);
bool end_file_list(const std::string& arg);

//%%%%%%%% Evaluation (extract an integer) %%%%%%%%

// Action: look for "Output written on ...", extract and return
//   number of pages in dvi output.
int page_count(const std::string&);


//%%%%%%%% Summaries %%%%%%%%

// Select and process log file lines for WWer's log report
// UPDATE criteria to collect additional data
std::string underfull(const std::string& arg); // "Underfull..."
std::string  overfull(const std::string& arg); // "Overfull..."
std::string latex_err(const std::string& arg); // "LaTeX Error:..."
std::string missing_pkg(const std::string& arg); // "LaTeX Error: File..."
